// OpenGL Tutorial
// Circle_List.c

/*************************************************************************
This example illustrates the use of display lists.
*************************************************************************/

// gcc -o Circle_List  Circle_List.c -lX11 -lMesaGL -lMesaGLU -lMesatk -lm

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <gltk.h>

#define My_Circle 1
#define PI 3.1415926535898

void reshape(int width, int height) {

	// Set the new viewport size
	glViewport(0, 0, (GLint)width, (GLint)height);

	// Choose the projection matrix to be the matrix 
	// manipulated by the following calls
	glMatrixMode(GL_PROJECTION);

	// Set the projection matrix to be the identity matrix
	glLoadIdentity();

	// Define the dimensions of the Orthographic Viewing Volume
	glOrtho(-8.0, 8.0, -8.0, 8.0, -8.0, 8.0);

	// Choose the modelview matrix to be the matrix
	// manipulated by further calls
	glMatrixMode(GL_MODELVIEW);
}

GLenum key_down(int key, GLenum state) {

	if ((key == TK_ESCAPE) || (key == TK_q) || (key == TK_Q))
		tkQuit();
}

void draw(void) {

	// Clear the the RGB buffer and the Depth buffer
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	// Set the modelview matrix to be the identity matrix
	glLoadIdentity();
	// Translate the object
	glTranslatef(-3.5, 3.5, 0.0);
	glScalef(2.0, 2.0, 1.0);
	glColor3f(1.0, 0.0, 1.0);

	// Draw the predefined circle
	glCallList(My_Circle);

	glLoadIdentity();
	glTranslatef(3.5, 3.5, 0.0);
	glRotatef(45, 1.0, 0.0, 0.0);
	glColor3f(0.0, 0.0, 1.0);

	// Draw the predefined circle
	glCallList(My_Circle);

	glLoadIdentity();
	glTranslatef(-3.5, -3.5, 0.0);
	glRotatef(45, 1.0, 0.0, 0.0);
	glRotatef(45, 0.0, 1.0, 0.0);
	glRotatef(45, 0.0, 0.0, 1.0);
	glScalef(2.0, 1.0, 1.0);
	glColor3f(0.0, 1.0, 0.0);

	// Draw the predefined circle
	glCallList(My_Circle);

	glLoadIdentity();
	glTranslatef(3.5, -3.5, 0.0);
	glScalef(1.0, 4.0, 1.0);
	glColor3f(0.0, 1.0, 1.0);

	// Draw the predefined circle
	glCallList(My_Circle);

	// Flush the buffer to force drawing of all objects thus far
	glFlush();
}


void make_circle() {

	GLuint i;
	GLfloat	cosine, sine;

	// Create a display list for a circle
	glNewList(My_Circle, GL_COMPILE);
		glBegin(GL_POLYGON);
			// Generate the points of the circle
			for(i = 0; i < 100; i++) {
				cosine = cos(i * 2 * PI / 100.0);
				sine = sin(i * 2 * PI/100.0);
				glVertex2f(cosine, sine);
			}
		glEnd();

	// End definition of circle
	glEndList();
}


void main(int argc, char **argv) {

	// Set top left corner of window to be at location (0, 0)
	// Set the window size to be 500x500 pixels
	tkInitPosition(0, 0, 500, 500);

	// Initialize the RGB and Depth buffers
	tkInitDisplayMode(TK_RGB | TK_DEPTH);

	// Open a window, name it "Circle_List"
	if (tkInitWindow("Circle_List") == GL_FALSE) {
		tkQuit();
	}

	// Set the clear color to black
	glClearColor(0.0, 0.0, 0.0, 0.0);

	// Set the shade model
	glShadeModel(GL_FLAT);

	// Set the polygon mode to fill
	glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);

	// Enable depth testing for hidden line removal
	glEnable(GL_DEPTH_TEST);

	// Call procedure to create the display list for the circle
	make_circle();

	// Assign reshape() to be the function called whenever
	// an expose event occurs
	tkExposeFunc(reshape);

	// Assign reshape() to be the function called whenever 
	// a reshape event occurs
	tkReshapeFunc(reshape);

	// Assign key_down() to be the function called whenever
	// a key is pressed
	tkKeyDownFunc(key_down);

	// Assign draw() to be the function called whenever a display
	// event occurs, generally after a resize or expose event
	tkDisplayFunc(draw);

	// Pass program control to tk's event handling code
	// In other words, loop forever
	tkExec();
}
